﻿using UnityEngine;
using System.Collections;

public class CharacterMoviment2D : MonoBehaviour {

	private Rigidbody2D m_Rigidbody2D;
	private Animator m_Anim;
	[SerializeField] private float m_MaxSpeed_Horizontal = 10;
	[SerializeField] private float m_MaxSpeed_Vertical = 10;
	private bool m_FacingRight = true;

	public bool rotate;

	// Use this for initialization
	void Start () {
		m_Rigidbody2D = GetComponent<Rigidbody2D> ();
		m_Anim = GetComponent<Animator> ();
	}

	public void Move(float move_horizontal, float move_vertical, bool crouch, bool jump) {
		// The Speed animator parameter is set to the absolute value of the horizontal input.
		//m_Anim.SetFloat("Speed", Mathf.Abs(move));

		// Move the character
		m_Rigidbody2D.velocity = new Vector2(move_horizontal*m_MaxSpeed_Horizontal, move_vertical*m_MaxSpeed_Vertical);

		if (rotate) {
			if (move_horizontal > 0) {
				transform.rotation = Quaternion.Euler (0,0,180);
			}

			if (move_horizontal < 0) {
				transform.rotation = Quaternion.Euler (0,0,0);
			}

			if (move_vertical > 0) {
				transform.rotation = Quaternion.Euler (0,0,-90);
			}

			if (move_vertical < 0) {
				transform.rotation = Quaternion.Euler (0,0,-270);
			}

			if (move_vertical < 0 && move_horizontal > 0) {
				transform.rotation = Quaternion.Euler (0,0,135);
			}

			if (move_vertical > 0 && move_horizontal < 0) {
				transform.rotation = Quaternion.Euler (0,0,-45);
			}

			if (move_vertical > 0 && move_horizontal > 0) {
				transform.rotation = Quaternion.Euler (0,0,-135);
			}

			if (move_vertical < 0 && move_horizontal < 0) {
				transform.rotation = Quaternion.Euler (0,0,45);
			}

		} 
		else {
			
			// If the input is moving the player right and the player is facing left...
			if (move_horizontal > 0 && !m_FacingRight) {
				// ... flip the player.
				Flip ();
			}
		// Otherwise if the input is moving the player left and the player is facing right...
			else if (move_horizontal < 0 && m_FacingRight) {
				// ... flip the player.
				Flip ();
			}
		}
	}

	public void MoveToOrigin(Vector3 point) {
		StartCoroutine (GoToPoint (point));
	}

	private void Flip()
	{
		// Switch the way the player is labelled as facing.
		m_FacingRight = !m_FacingRight;

		// Multiply the player's x local scale by -1.
		//Vector3 theScale = transform.localScale;
		//theScale.x *= -1;
		//transform.localScale = theScale;

		GetComponent<SpriteRenderer> ().flipX = !GetComponent<SpriteRenderer> ().flipX;
	}

	public bool RbKinematic {
		get { return m_Rigidbody2D.isKinematic; }
		set { m_Rigidbody2D.isKinematic = value; }
	}

	IEnumerator GoToPoint(Vector3 point) {
		GetComponent<BoxCollider2D> ().enabled = false;

		while (transform.position != point) {

			transform.position = Vector3.MoveTowards (transform.position, point, (m_MaxSpeed_Vertical * 0.6f) * Time.deltaTime);

			if (rotate) {
				transform.rotation = Quaternion.Euler (0,0,135);
			}

			yield return null;
		}

		if (rotate) {
			transform.rotation = Quaternion.Euler (0,0,0);
		}

		GetComponent<BoxCollider2D> ().enabled = true;
	}
}
