﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.SceneManagement;

public class GuardControler2D : MonoBehaviour {

	public enum States {
		guarding,
		chasing,
		beatPlayer,
		actuallyBeatPlayer
	}

	public LayerMask mask, mask2;
	public States state;
	public List<GameObject> targets = new List<GameObject> ();

	public GameObject roomTrigger;
	public GameObject bedTrigger;
	public GameObject blackScreen;

	public Sprite back, normal;

	GuardMoviment2D m_Guard;
	SpriteRenderer m_sr;
	private bool locked, round, makingRound;
	private int currentCell;
	private float roundTime = 6;

	public GameObject rat;

	bool end;

	// Use this for initialization
	void Start () {
		m_Guard = GetComponent<GuardMoviment2D> ();
		m_sr = GetComponent<SpriteRenderer> ();
	}

	// Update is called once per frame
	void Update () {

	}

	void FixedUpdate () {
		rat = GameObject.FindGameObjectWithTag ("rat");

		//print (Vector3.Distance (rat.transform.position, transform.position));

		if (state == States.guarding) {
			LookForRat ();

			//check if has reached target
			if (transform.position == targets [currentCell].transform.position) {
				//yes? stop & make round
				if (!makingRound) {
					makingRound = true;
					StartCoroutine (MakeRound ());
				}


				//after round is done, where to?
				if (!round) {
					currentCell++;

					if (currentCell >= targets.Count)
						currentCell = 0;

					makingRound = false;
				}
				
			} 
			//no? keep moving
		else {
				//tell where to move	
				m_Guard.Move (targets [currentCell]);
			}
		}
		else if (state == States.beatPlayer) {
			if (transform.position != targets[0].transform.position) {
				m_Guard.Move (targets [0]);
			} else {
				//send raycast
				RaycastHit2D hit = Physics2D.Raycast(transform.position, Vector2.up, 0.2f, mask);

				if(hit.collider != null) {
					print (hit.collider.transform.tag);
					print (hit.collider.transform.name);

					if (hit.collider.transform.tag == "CellDoor") {
						hit.collider.transform.parent.GetComponent<CellScript> ().CallOpenCell ();
						StopCoroutine (MakeRound ());
						StartCoroutine (FadeOut ());
						state = States.actuallyBeatPlayer;
					}
				}
			}
		}
		else if (state == States.actuallyBeatPlayer) {
			m_Guard.MoveForward ();

			Destroy (GameObject.Find ("Canvas"));

			if (!end) {
				StartCoroutine (WaitToEnd ());
				end = true;
			}
		}

		else if (state == States.chasing) {
			if (rat != null) {

				Vector3 dir = (rat.transform.position - transform.position).normalized;
				RaycastHit2D hit = Physics2D.Raycast (transform.position, dir, 10, mask2);

				if (hit.collider.transform.tag == "rat") {
					m_Guard.Move (rat);
				} else {
					state = States.guarding;
				}
			} else {
				state = States.guarding;
			}
		}
	}

	void LookForRat() {
		if (rat != null) {

			Vector3 dir = (rat.transform.position - transform.position).normalized;

			RaycastHit2D hit = Physics2D.Raycast(transform.position, dir, 10, mask2);

			if (hit.collider.transform.tag == "rat") {
				if (!round) {
					if (hit.collider.transform.position.x > transform.position.x && m_Guard.FacingRight) {
						print ("looking at rat right");

						state = States.chasing;
					}

					if (hit.collider.transform.position.x < transform.position.x && !m_Guard.FacingRight) {
						print ("looking at rat left");

						state = States.chasing;
					}
				} else {
					if (hit.collider.transform.position.y > transform.position.y && Mathf.Abs(Vector3.Distance (hit.collider.transform.position, transform.position)) < 1.2f) {
						print ("looking at rat front");

						state = States.chasing;
					}
				}
			}
		}
	}

	public bool Locked {
		get { return locked; }
		set { locked = value; }
	}

	IEnumerator MakeRound() {
		float timer = 0;

		round = true;

		while (timer < roundTime) {

			m_sr.sprite = back;

//			if (timer <= 1) {
//				if (m_Guard.FacingRight) {
//					transform.rotation = Quaternion.RotateTowards (transform.rotation, Quaternion.Euler (new Vector3 (0, 0, 90)), 90 * Time.deltaTime);	
//				} else {
//					transform.rotation = Quaternion.RotateTowards (transform.rotation, Quaternion.Euler (new Vector3 (0, 0, -90)), 90 * Time.deltaTime);	
//				}
//			}
//
//			if (timer >= roundTime - 1) {
//				if (m_Guard.FacingRight) {
//					transform.rotation = Quaternion.RotateTowards (transform.rotation, Quaternion.Euler (new Vector3 (0, 0, 0)), 90 * Time.deltaTime);	
//				} else {
//					transform.rotation = Quaternion.RotateTowards (transform.rotation, Quaternion.Euler (new Vector3 (0, 0, 0)), 90 * Time.deltaTime);	
//				}
//			}

			if (currentCell == 0 && state == States.guarding) {
				//check if player is in room
				if (!roomTrigger.GetComponent<RoomTrigger> ().playerInRoom) {
					print ("player not in the room");
							
					state = States.beatPlayer;
				}
				//check if bed is in place
				if (!bedTrigger.GetComponent<BedTrigger> ().bedInPlace  && state == States.guarding) {
					print ("bed not in place");

					state = States.beatPlayer;
				}
			}

			timer += Time.deltaTime;

			yield return null;
		}

		//transform.rotation = Quaternion.Euler (Vector3.zero);

		m_sr.sprite = normal;

		round = false;
	}

	IEnumerator FadeOut() {

		Color c = blackScreen.GetComponent<SpriteRenderer> ().color;
		float fade = 0;

		while (fade <= 1) {
			c.a = fade;
			blackScreen.GetComponent<SpriteRenderer> ().color = c;

			fade += Time.deltaTime * 1;

			yield return null;
		}
	}

	IEnumerator WaitToEnd() {
		yield return new WaitForSeconds (4);

		SceneManager.LoadScene (0);
	}
}
